from datetime import datetime
from lxml import etree, objectify


def parse_xml(value, tag_str_lower):
    if "float" in tag_str_lower:
        return float(value)
    elif "date" in tag_str_lower:
        return datetime.fromisoformat(value)
    return str(value)


def apply(input_path, parameters=None):
    if parameters is None:
        parameters = {}

    parser = etree.XMLParser(remove_comments=True)
    tree = objectify.parse(input_path, parser=parser)
    root = tree.getroot()

    log = {}
    log["ocel:version"] = root.get("ocel-xml.version")
    log["ocel:events"] = {}
    log["ocel:objects"] = {}

    for child in root:
        if child.tag.lower().endswith("global"):
            scope = child.get("scope")
            if scope == "event":
                log["ocel:global-event"] = {}
                for child2 in child:
                    log["ocel:global-event"][child2.get("key")] = child2.get("value")
            elif scope == "object":
                log["ocel:global-object"] = {}
                for child2 in child:
                    log["ocel:global-object"][child2.get("key")] = child2.get("value")
            elif scope == "log":
                log["ocel:global-log"] = {}
                for child2 in child:
                    if child2.get("key") == "attribute-names":
                        log["ocel:global-log"]["ocel:attribute-names"] = []
                        for child3 in child2:
                            log["ocel:global-log"]["ocel:attribute-names"].append(child3.get("value"))
                    elif child2.get("key") == "object-types":
                        log["ocel:global-log"]["ocel:object-types"] = []
                        for child3 in child2:
                            log["ocel:global-log"]["ocel:object-types"].append(child3.get("value"))

        if child.tag.lower().endswith("events"):
            for event in child:
                eve = {}
                for child2 in event:
                    if child2.get("key") == "id":
                        eve["ocel:id"] = child2.get("value")
                    elif child2.get("key") == "timestamp":
                        eve["ocel:timestamp"] = datetime.fromisoformat(child2.get("value"))
                    elif child2.get("key") == "activity":
                        eve["ocel:activity"] = child2.get("value")
                    elif child2.get("key") == "omap":
                        omap = []
                        for child3 in child2:
                            omap.append(child3.get("value"))
                        eve["ocel:omap"] = omap
                    elif child2.get("key") == "vmap":
                        eve["ocel:vmap"] = {}
                        for child3 in child2:
                            eve["ocel:vmap"][child3.get("key")] = parse_xml(child3.get("value"), child3.tag.lower())
                log["ocel:events"][eve["ocel:id"]] = eve
                del eve["ocel:id"]
        elif child.tag.lower().endswith("objects"):
            for object in child:
                obj = {}
                for child2 in object:
                    if child2.get("key") == "id":
                        obj["ocel:id"] = child2.get("value")
                    elif child2.get("key") == "type":
                        obj["ocel:type"] = child2.get("value")
                    elif child2.get("key") == "ovmap":
                        obj["ocel:ovmap"] = {}
                        for child3 in child2:
                            obj["ocel:ovmap"][child3.get("key")] = parse_xml(child3.get("value"), child3.tag.lower())
                log["ocel:objects"][obj["ocel:id"]] = obj
                del obj["ocel:id"]

    return log



from ocel.exporter import oceljson, ocelxml


def apply(log, output_path, parameters=None):
    if ".json" in output_path:
        return oceljson.apply(log, output_path, parameters=parameters)
    else:
        return ocelxml.apply(log, output_path, parameters=parameters)



import lxml


def apply(input_path, validation_path, parameters=None):
    xml_file = lxml.etree.parse(input_path)
    xml_validator = lxml.etree.XMLSchema(file=validation_path)
    is_valid = xml_validator.validate(xml_file)
    return is_valid



from ocel import exporter, importer, validation
from ocel.interface import *


__version__ = '0.0.2'
__doc__ = "OCEL (object-centric event log) support for Python"
__author__ = 'PADS'
__author_email__ = 'a.berti@pads.rwth-aachen.de'
__maintainer__ = 'PADS'
__maintainer_email__ = "a.berti@pads.rwth-aachen.de"


def import_log(log_path):
    return importer.importer.apply(log_path)


def export_log(log, log_path):
    return exporter.exporter.apply(log, log_path)


def validate(log_path, schema_path):
    return validation.validate.apply(log_path, schema_path)



import json
from datetime import datetime


def apply(input_path, parameters=None):
    if parameters is None:
        parameters = {}
    log_obj = json.load(open(input_path, "rb"))
    for ek in log_obj["ocel:events"]:
        eve = log_obj["ocel:events"][ek]
        eve["ocel:timestamp"] = datetime.fromisoformat(eve["ocel:timestamp"])
    return log_obj



def get_events(log):
    return log["ocel:events"]


def get_objects(log):
    return log["ocel:objects"]


def get_attribute_names(log):
    return log["ocel:global-log"]["ocel:attribute-names"]


def get_object_types(log):
    return log["ocel:global-log"]["ocel:object-types"]


def get_version(log):
    return log["ocel:version"]


def get_global_event(log):
    return log["ocel:global-event"]


def get_global_object(log):
    return log["ocel:global-object"]



from ocel.validation import oceljson, ocelxml


def apply(input_path, validation_path, parameters=None):
    if ".json" in input_path:
        return oceljson.apply(input_path, validation_path, parameters=parameters)
    else:
        return ocelxml.apply(input_path, validation_path, parameters=parameters)



from lxml import etree


def get_type(t0):
    if "float" in str(t0).lower() or "double" in str(t0).lower():
        return "float"
    elif "object" in str(t0).lower():
        return "string"
    else:
        return "string"


def apply(log, output_path, parameters=None):
    if parameters is None:
        parameters = {}

    prefix = "ocel:"

    root = etree.Element("log")

    if "ocel:version" in log:
        root.set("ocel-xml.version", str(log["ocel:version"]))

    global_event = etree.SubElement(root, "global")
    global_event.set("scope", "event")
    for k, v in log[prefix +"global-event"].items():
        child = etree.SubElement(global_event, "string")
        child.set("key", k.split(prefix)[-1])
        child.set("value", v)

    global_object = etree.SubElement(root, "global")
    global_object.set("scope", "event")

    for k, v in log[prefix +"global-object"].items():
        child = etree.SubElement(global_object, "string")
        child.set("key", k.split(prefix)[-1])
        child.set("value", v)

    global_log = etree.SubElement(root, "global")
    global_log.set("scope", "log")
    attribute_names = etree.SubElement(global_log, "list")
    attribute_names.set("key", "attribute-names")
    object_types = etree.SubElement(global_log, "list")
    object_types.set("key", "object-types")
    for k in log[prefix +"global-log"][prefix +"attribute-names"]:
        subel = etree.SubElement(attribute_names, "string")
        subel.set("key", "attribute-name")
        subel.set("value", k)
    for k in log[prefix +"global-log"][prefix +"object-types"]:
        subel = etree.SubElement(object_types, "string")
        subel.set("key", "object-type")
        subel.set("value", k)
    events = etree.SubElement(root, "events")
    for k, v in log[prefix + "events"].items():
        event = etree.SubElement(events, "event")
        event_id = etree.SubElement(event, "string")
        event_id.set("key", "id")
        event_id.set("value", str(k))
        event_activity = etree.SubElement(event, "string")
        event_activity.set("key", "activity")
        event_activity.set("value", v[prefix+"activity"])
        event_timestamp = etree.SubElement(event, "date")
        event_timestamp.set("key", "timestamp")
        event_timestamp.set("value", v[prefix+"timestamp"].isoformat())
        event_omap = etree.SubElement(event, "list")
        event_omap.set("key", "omap")
        for k2 in v[prefix+"omap"]:
            obj = etree.SubElement(event_omap, "string")
            obj.set("key", "object-id")
            obj.set("value", k2)
        event_vmap = etree.SubElement(event, "list")
        event_vmap.set("key", "vmap")
        for k2, v2 in v[prefix+"vmap"].items():
            attr = etree.SubElement(event_vmap, get_type(v2))
            attr.set("key", k2)
            attr.set("value", str(v2))
    objects = etree.SubElement(root, "objects")
    for k, v in log[prefix+"objects"].items():
        object = etree.SubElement(objects, "object")
        object_id = etree.SubElement(object, "string")
        object_id.set("key", "id")
        object_id.set("value", str(k))
        object_type = etree.SubElement(object, "string")
        object_type.set("key", "type")
        object_type.set("value", v[prefix+"type"])
        object_ovmap = etree.SubElement(object, "list")
        object_ovmap.set("key", "ovmap")
        for k2, v2 in v[prefix+"ovmap"].items():
            if str(v2).lower() != "nan" and str(v2).lower() != "nat":
                object_att = etree.SubElement(object_ovmap, get_type(v2))
                object_att.set("key", k2)
                object_att.set("value", str(v2))

    tree = etree.ElementTree(root)
    tree.write(output_path, pretty_print=True, xml_declaration=True, encoding="utf-8")



import datetime
import json


def myconverter(o):
    if isinstance(o, datetime.datetime):
        return o.isoformat()


def apply(log, output_path, parameters=None):
    if parameters is None:
        parameters = {}

    json.dump(log, open(output_path, "w"), indent=2, default=myconverter)



import json
import jsonschema
from jsonschema import validate


def apply(input_path, validation_path, parameters=None):
    if parameters is None:
        parameters = {}
    file_content = json.load(open(input_path, "rb"))
    schema_content = json.load(open(validation_path, "rb"))
    try:
        validate(instance=file_content, schema=schema_content)
        return True
    except jsonschema.exceptions.ValidationError as err:
        return False



from os.path import dirname, join

import ocel
from setuptools import setup


def read_file(filename):
    with open(join(dirname(__file__), filename)) as f:
        return f.read()


setup(
    name="ocel-standard",
    version=ocel.__version__,
    description=ocel.__doc__.strip(),
    long_description=read_file('README.md'),
    author=ocel.__author__,
    author_email=ocel.__author_email__,
    py_modules=[ocel.__name__],
    include_package_data=True,
    packages=['ocel', 'ocel.exporter', 'ocel.importer', 'ocel.validation'],
    url='http://www.pm4py.org',
    license='GPL-3.0',
    install_requires=[
        "jsonschema",
        "lxml"
    ]
)
